@@ -1,7 +1,7 @@
 /*
  * misc.c	Various miscellaneous functions.
  *
- * Version:	$Id: 8a9789275ecc77205b12f91cb7c256870d4c605d $
+ * Version:	$Id: 36378ec40c9387da9b4c4c1081aeaa93e59a7142 $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 8a9789275ecc77205b12f91cb7c256870d4c605d $")
+RCSID("$Id: 36378ec40c9387da9b4c4c1081aeaa93e59a7142 $")
 
 #include <freeradius-devel/libradius.h>
 
@@ -607,13 +607,6 @@ int fr_pton(fr_ipaddr_t *out, char const *value, ssize_t inlen, int af, bool res
 			fr_strerror_printf("Invalid address");
 			return -1;
 		}
-
-		/*
-		 *	Fall through to resolving the address, using
-		 *	whatever address family they prefer.  If they
-		 *	don't specify an address family, force IPv4.
-		 */
-		if (af == AF_UNSPEC) af = AF_INET;
 	}
 
 	/*
@@ -815,10 +808,10 @@ int rad_unlockfd(int fd, int lock_len)
 	fl.l_start = 0;
 	fl.l_len = lock_len;
 	fl.l_pid = getpid();
-	fl.l_type = F_WRLCK;
+	fl.l_type = F_UNLCK;
 	fl.l_whence = SEEK_CUR;
 
-	return fcntl(fd, F_UNLCK, (void *)&fl);
+	return fcntl(fd, F_SETLK, (void *)&fl);
 #else
 #error "missing definition for F_WRLCK, all file locks will fail"
 
@@ -871,7 +864,7 @@ uint8_t *ifid_aton(char const *ifid_str, uint8_t *ifid)
 			num_id = 0;
 			if ((idx += 2) > 6)
 				return NULL;
-		} else if ((pch = strchr(xdigits, tolower(*p))) != NULL) {
+		} else if ((pch = strchr(xdigits, tolower((uint8_t) *p))) != NULL) {
 			if (++num_id > 4)
 				return NULL;
 			/*
@@ -1332,8 +1325,8 @@ size_t fr_hex2bin(uint8_t *bin, size_t outlen, char const *hex, size_t inlen)
 	if (len > outlen) len = outlen;
 
 	for (i = 0; i < len; i++) {
-		if(!(c1 = memchr(hextab, tolower((int) hex[i << 1]), sizeof(hextab))) ||
-		   !(c2 = memchr(hextab, tolower((int) hex[(i << 1) + 1]), sizeof(hextab))))
+		if(!(c1 = memchr(hextab, tolower((uint8_t) hex[i << 1]), sizeof(hextab))) ||
+		   !(c2 = memchr(hextab, tolower((uint8_t) hex[(i << 1) + 1]), sizeof(hextab))))
 			break;
 		bin[i] = ((c1-hextab)<<4) + (c2-hextab);
 	}
@@ -1410,8 +1403,9 @@ uint32_t fr_strtoul(char const *value, char **end)
 bool is_whitespace(char const *value)
 {
 	do {
-		if (!isspace(*value)) return false;
-	} while (*++value);
+		if (!isspace((uint8_t) *value)) return false;
+		value++;
+	} while (*value);
 
 	return true;
 }
@@ -1446,9 +1440,21 @@ bool is_whitespace(char const *value)
  */
 bool is_integer(char const *value)
 {
+#ifndef __clang_analyzer__
 	do {
-		if (!isdigit(*value)) return false;
-	} while (*++value);
+		if (!isdigit((uint8_t) *value)) return false;
+		value++;
+	} while (*value);
+
+	/*
+	 *	Clang analyzer complains about the above line: "Branch
+	 *	depends on a garbage value", even though that's
+	 *	clearly not true.  And, it doesn't complain about the
+	 *	other functions doing similar things.
+	 */
+#else
+	if (!isdigit((uint8_t) *value)) return false;
+#endif
 
 	return true;
 }
@@ -1461,7 +1467,8 @@ bool is_zero(char const *value)
 {
 	do {
 		if (*value != '0') return false;
-	} while (*++value);
+		value++;
+	} while (*value);
 
 	return true;
 }
@@ -2147,6 +2154,30 @@ void fr_quick_sort(void const *to_sort[], int min_idx, int max_idx, fr_cmp_t cmp
 	fr_quick_sort(to_sort, part + 1, max_idx, cmp);
 }
 
+#define USEC 1000000
+
+/** Convert a time specified in milliseconds to a timeval
+ *
+ * @param[out] out	Where to write the result.
+ * @param[in] ms	To convert to a timeval struct.
+ */
+void fr_timeval_from_ms(struct timeval *out, uint64_t ms)
+{
+	out->tv_sec = ms / 1000;
+	out->tv_usec = (ms % 1000) * 1000;
+}
+
+/** Convert a time specified in microseconds to a timeval
+ *
+ * @param[out] out	Where to write the result.
+ * @param[in] usec	To convert to a timeval struct.
+ */
+void fr_timeval_from_usec(struct timeval *out, uint64_t usec)
+{
+	out->tv_sec = usec / USEC;
+	out->tv_usec = usec % USEC;
+}
+
 #ifdef TALLOC_DEBUG
 void fr_talloc_verify_cb(UNUSED const void *ptr, UNUSED int depth,
 			 UNUSED int max_depth, UNUSED int is_ref,
