@@ -1,7 +1,7 @@
 /*
  * pair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: 898e1e85296bfa0a8a89976f2329f123054eac12 $
+ * Version:	$Id: c86deb8a142c0fb366b603993c5e8e0760c1debf $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 898e1e85296bfa0a8a89976f2329f123054eac12 $")
+RCSID("$Id: c86deb8a142c0fb366b603993c5e8e0760c1debf $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/regex.h>
@@ -1795,7 +1795,7 @@ FR_TOKEN fr_pair_raw_from_str(char const **ptr, VALUE_PAIR_RAW *raw)
 		 *	=
 		 *	value
 		 */
-		if ((*p == ':') && (!isdigit((int) p[1]))) {
+		if ((*p == ':') && (!isdigit((uint8_t) p[1]))) {
 			break;
 		}
 
@@ -1814,13 +1814,13 @@ FR_TOKEN fr_pair_raw_from_str(char const **ptr, VALUE_PAIR_RAW *raw)
 	 *	Look for tag (:#).  This is different from :=, which
 	 *	is an operator.
 	 */
-	if ((*p == ':') && (isdigit((int) p[1]))) {
+	if ((*p == ':') && (isdigit((uint8_t) p[1]))) {
 		if (q >= (raw->l_opand + sizeof(raw->l_opand))) {
 			goto too_long;
 		}
 		*(q++) = *(p++);
 
-		while (isdigit((int) *p)) {
+		while (isdigit((uint8_t) *p)) {
 			if (q >= (raw->l_opand + sizeof(raw->l_opand))) {
 				goto too_long;
 			}
