@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: a8c667bfb5df142f6bfde864339510f2ef3920e4 $
+ * Version:	$Id: 8a9297b5d6b6e8f039fa475364b0e1c00fb2c270 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: a8c667bfb5df142f6bfde864339510f2ef3920e4 $")
+RCSID("$Id: 8a9297b5d6b6e8f039fa475364b0e1c00fb2c270 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
@@ -127,7 +127,7 @@ typedef struct cf_file_t {
 } cf_file_t;
 
 CONF_SECTION *root_config = NULL;
-bool cf_new_escape = false;
+bool cf_new_escape = true;
 
 
 static int		cf_data_add_internal(CONF_SECTION *cs, char const *name, void *data,
@@ -1174,10 +1174,14 @@ static char const *cf_expand_variables(char const *cf, int *lineno,
 					return NULL;
 				}
 
+				/*
+				 *	Might as well make
+				 *	non-existent string be the
+				 *	empty string.
+				 */
 				if (!cp->value) {
-					ERROR("%s[%d]: Reference \"%s\" has no value",
-					       cf, *lineno, input);
-					return NULL;
+					*p = '\0';
+					goto skip_value;
 				}
 
 				if (p + strlen(cp->value) >= output + outsize) {
@@ -1188,6 +1192,7 @@ static char const *cf_expand_variables(char const *cf, int *lineno,
 
 				strcpy(p, cp->value);
 				p += strlen(p);
+			skip_value:
 				ptr = end + 1;
 
 			} else if (ci->type == CONF_ITEM_SECTION) {
@@ -2313,7 +2318,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 
 		if (has_spaces) {
 			ptr = cbuf;
-			while (isspace((int) *ptr)) ptr++;
+			while (isspace((uint8_t) *ptr)) ptr++;
 
 			if (ptr > cbuf) {
 				memmove(cbuf, ptr, len - (ptr - cbuf));
@@ -2329,7 +2334,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 			if (at_eof) break;
 
 			ptr = buf;
-			while (*ptr && isspace((int) *ptr)) ptr++;
+			while (*ptr && isspace((uint8_t) *ptr)) ptr++;
 
 			if (!*ptr || (*ptr == '#')) continue;
 
@@ -2511,6 +2516,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 				 */
 				while ((dp = readdir(dir)) != NULL) {
 					char const *p;
+					int slen;
 
 					if (dp->d_name[0] == '.') continue;
 
@@ -2518,8 +2524,8 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 					 *	Check for valid characters
 					 */
 					for (p = dp->d_name; *p != '\0'; p++) {
-						if (isalpha((int)*p) ||
-						    isdigit((int)*p) ||
+						if (isalpha((uint8_t)*p) ||
+						    isdigit((uint8_t)*p) ||
 						    (*p == '-') ||
 						    (*p == '_') ||
 						    (*p == '.')) continue;
@@ -2527,8 +2533,12 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 					}
 					if (*p != '\0') continue;
 
-					snprintf(buf2, sizeof(buf2), "%s%s",
-						 value, dp->d_name);
+					slen = snprintf(buf2, sizeof(buf2), "%s%s",
+							value, dp->d_name);
+					if (slen >= (int) sizeof(buf2) || slen < 0) {
+						ERROR("%s: Full file path is too long.", dp->d_name);
+						return -1;
+					}
 					if ((stat(buf2, &stat_buf) != 0) ||
 					    S_ISDIR(stat_buf.st_mode)) continue;
 
@@ -2794,7 +2804,15 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 
 		case T_OP_EQ:
 		case T_OP_SET:
-			while (isspace((int) *ptr)) ptr++;
+		case T_OP_PREPEND:
+			while (isspace((uint8_t) *ptr)) ptr++;
+
+			/*
+			 *	Be a little more forgiving.
+			 */
+			if (*ptr == '#') {
+				t3 = T_HASH;
+			} else
 
 			/*
 			 *	New parser: non-quoted strings are
@@ -2809,7 +2827,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 
 				t3 = T_BARE_WORD;
 				while (*q && (*q >= ' ') && (*q != ',') &&
-				       !isspace(*q)) q++;
+				       !isspace((uint8_t) *q)) q++;
 
 				if ((size_t) (q - ptr) >= sizeof(buf3)) {
 					ERROR("%s[%d]: Parse error: value too long",
@@ -2877,21 +2895,10 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 			cpn->pass2 = pass2;
 			cf_item_add(this, &(cpn->item));
 
-			/*
-			 *	Hacks for escaping
-			 */
-			if (!cf_new_escape && !this->item.parent && value &&
-			    (strcmp(buf1, "correct_escapes") == 0) &&
-			    ((strcmp(value, "true") == 0) ||
-			     (strcmp(value, "yes") == 0) ||
-			     (strcmp(value, "1") == 0))) {
-				cf_new_escape = true;
-			}
-
 			/*
 			 *	Require a comma, unless there's a comment.
 			 */
-			while (isspace(*ptr)) ptr++;
+			while (isspace((uint8_t) *ptr)) ptr++;
 
 			if (*ptr == ',') {
 				ptr++;
@@ -2964,7 +2971,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 		/*
 		 *	Done parsing one thing.  Skip to EOL if possible.
 		 */
-		while (isspace(*ptr)) ptr++;
+		while (isspace((uint8_t) *ptr)) ptr++;
 
 		if (*ptr == '#') continue;
 
@@ -3561,6 +3568,10 @@ bool cf_item_is_pair(CONF_ITEM const *item)
 	return item->type == CONF_ITEM_PAIR;
 }
 
+bool cf_item_is_data(CONF_ITEM const *item)
+{
+	return item->type == CONF_ITEM_DATA;
+}
 
 static CONF_DATA *cf_data_alloc(CONF_SECTION *parent, char const *name,
 				void *data, void (*data_free)(void *))
