@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: cd0b912938eb7b433cf369a24e88fb3a473f03ad $
+ * Version:	$Id: 774affce29976d65db4a016363f32db8a312c43b $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: cd0b912938eb7b433cf369a24e88fb3a473f03ad $")
+RCSID("$Id: 774affce29976d65db4a016363f32db8a312c43b $")
 USES_APPLE_DEPRECATED_API	/* OpenSSL API has been deprecated by Apple */
 
 #include <freeradius-devel/radiusd.h>
@@ -758,7 +758,7 @@ static void *request_handler_thread(void *arg)
 			 *	waiting, but DON'T print out any more
 			 *	text.
 			 */
-			if (errno == EINTR) {
+			if ((errno == EINTR) || (errno == EAGAIN)) {
 				DEBUG2("Re-wait %d", self->thread_num);
 				goto re_wait;
 			}
@@ -1576,6 +1576,29 @@ void thread_pool_queue_stats(int array[RAD_LISTEN_MAX], int pps[2])
 		pps[0] = pps[1] = 0;
 	}
 }
+
+void thread_pool_thread_stats(int stats[3])
+{
+#ifndef WITH_GCD
+	if (pool_initialized) {
+		/*
+		 *	We don't need a mutex lock here as we only want to
+		 *	read a close approximation of the number of active
+		 *	threads, and not modify it.
+		 */
+#ifdef HAVE_STDATOMIC_H
+		stats[0] = load(thread_pool.active_threads);
+#else
+		stats[0] = thread_pool.active_threads;
+#endif
+		stats[1] = thread_pool.total_threads;
+		stats[2] = thread_pool.max_threads;
+	} else
+#endif	/* WITH_GCD */
+	{
+		stats[0] = stats[1] = stats[2] = 0;
+	}
+}
 #endif /* HAVE_PTHREAD_H */
 
 static void time_free(void *data)
