@@ -1,7 +1,7 @@
 /*
  * radeapclient.c	EAP specific radius packet debug tool.
  *
- * Version:	$Id: 553a6a6a57e074088b66506640af3e0efc17e36c $
+ * Version:	$Id: 955df1eae030110520222c084ed4bbe8ac41766f $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 553a6a6a57e074088b66506640af3e0efc17e36c $")
+RCSID("$Id: 955df1eae030110520222c084ed4bbe8ac41766f $")
 
 #include <freeradius-devel/libradius.h>
 
@@ -182,6 +182,14 @@ static void rc_get_port(PW_CODE type, uint16_t *port);
 static void rc_evprep_packet_timeout(rc_transaction_t *trans);
 static void rc_deallocate_id(rc_transaction_t *trans);
 
+/*
+ *	For cbtls_cache_*()
+ */
+rlm_rcode_t process_post_auth(UNUSED int postauth_type, UNUSED REQUEST *request)
+{
+	return RLM_MODULE_FAIL;
+}
+
 
 static void NEVER_RETURNS usage(void)
 {
@@ -1951,7 +1959,7 @@ int main(int argc, char **argv)
 		  break;
 
 		case 'r':
-			if (!isdigit((int) *optarg))
+			if (!isdigit((uint8_t) *optarg))
 				usage();
 			retries = atoi(optarg);
 			break;
@@ -1959,12 +1967,12 @@ int main(int argc, char **argv)
 			do_summary = 1;
 			break;
 		case 't':
-			if (!isdigit((int) *optarg))
+			if (!isdigit((uint8_t) *optarg))
 				usage();
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("$Id: 553a6a6a57e074088b66506640af3e0efc17e36c $"
+			printf("$Id: 955df1eae030110520222c084ed4bbe8ac41766f $"
 #ifndef ENABLE_REPRODUCIBLE_BUILDS
 			", built on " __DATE__ " at " __TIME__
 #endif
@@ -2051,7 +2059,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Get the request type
 	 */
-	if (!isdigit((int) argv[2][0])) {
+	if (!isdigit((uint8_t) argv[2][0])) {
 		packet_code = fr_str2int(rc_request_types, argv[2], -2);
 		if (packet_code == -2) {
 			ERROR("Unrecognised request type \"%s\"\n", argv[2]);
