@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 2959b6837cc4a6fe04e551c03a66ee8939f817db $
+ * $Id: 97fdcbe2f35d57c7f7e6bebbfc44a94c6da4304e $
  * @file ldap.c
  * @brief LDAP module library functions.
  *
@@ -1101,7 +1101,7 @@ char const *rlm_ldap_find_user(rlm_ldap_t const *inst, REQUEST *request, ldap_ha
 	 *	If the caller isn't looking for the result we can just return the current userdn value.
 	 */
 	if (!force) {
-		vp = fr_pair_find_by_num(request->config, PW_LDAP_USERDN, 0, TAG_ANY);
+		vp = fr_pair_find_by_da(request->config, inst->user_dn_da, TAG_ANY);
 		if (vp) {
 			RDEBUG("Using user DN from request \"%s\"", vp->vp_strvalue);
 			*rcode = RLM_MODULE_OK;
@@ -1213,8 +1213,9 @@ char const *rlm_ldap_find_user(rlm_ldap_t const *inst, REQUEST *request, ldap_ha
 	 *	we pass the string back to libldap we must not alter it.
 	 */
 	RDEBUG("User object found at DN \"%s\"", dn);
-	vp = fr_pair_make(request, &request->config, "LDAP-UserDN", NULL, T_OP_EQ);
+	vp = fr_pair_afrom_da(request, inst->user_dn_da);
 	if (vp) {
+		fr_pair_add(&request->config, vp);
 		fr_pair_value_strcpy(vp, dn);
 		*rcode = RLM_MODULE_OK;
 	}
@@ -1315,8 +1316,6 @@ static int rlm_ldap_rebind(LDAP *handle, LDAP_CONST char *url, UNUSED ber_tag_t
 
 	int ldap_errno;
 
-	conn->referred = true;
-	conn->rebound = true;	/* not really, but oh well... */
 	rad_assert(handle == conn->handle);
 
 	DEBUG("rlm_ldap (%s): Rebinding to URL %s", conn->inst->name, url);
@@ -1412,7 +1411,6 @@ void *mod_conn_create(TALLOC_CTX *ctx, void *instance)
 
 	conn->inst = inst;
 	conn->rebound = false;
-	conn->referred = false;
 
 	DEBUG("rlm_ldap (%s): Connecting to %s", inst->name, inst->server);
 #ifdef HAVE_LDAP_INITIALIZE
@@ -1514,12 +1512,18 @@ void *mod_conn_create(TALLOC_CTX *ctx, void *instance)
 	maybe_ldap_option(LDAP_OPT_X_TLS_CERTFILE, "certificate_file", inst->tls_certificate_file);
 	maybe_ldap_option(LDAP_OPT_X_TLS_KEYFILE, "private_key_file", inst->tls_private_key_file);
 
-#  ifdef LDAP_OPT_X_TLS_NEVER
+#  ifdef LDAP_OPT_X_TLS_REQUIRE_CERT
 	if (inst->tls_require_cert_str) {
 		do_ldap_option(LDAP_OPT_X_TLS_REQUIRE_CERT, "require_cert", &inst->tls_require_cert);
 	}
 #  endif
 
+#  ifdef LDAP_OPT_X_TLS_PROTOCOL_MIN
+	if (inst->tls_min_version_str) {
+		do_ldap_option(LDAP_OPT_X_TLS_PROTOCOL_MIN, "tls_min_version", &inst->tls_min_version);
+	}
+#  endif
+
 	/*
 	 *	Counter intuitively the TLS context appears to need to be initialised
 	 *	after all the TLS options are set on the handle.
@@ -1533,6 +1537,12 @@ void *mod_conn_create(TALLOC_CTX *ctx, void *instance)
 	}
 #  endif
 
+#  ifdef LDAP_OPT_X_TLS_CIPHER_SUITE
+	if (inst->tls_cipher_list) {
+		do_ldap_option(LDAP_OPT_X_TLS_CIPHER_SUITE, "cipher_list", inst->tls_cipher_list);
+	}
+#  endif
+
 	/*
 	 *	And finally start the TLS code.
 	 */
@@ -1596,21 +1606,6 @@ void mod_conn_release(rlm_ldap_t const *inst, ldap_handle_t *conn)
 	 */
 	if (!conn) return;
 
-	/*
-	 *	We chased a referral to another server.
-	 *
-	 *	This connection is no longer part of the pool which is
-	 *	connected to and bound to the configured server.
-	 *	Close it.
-	 *
-	 *	Note that we do NOT close it if it was bound to another user.
-	 *	Instead, we let the next caller do the rebind.
-	 */
-	if (conn->referred) {
-		fr_connection_close(inst->pool, conn, "Was referred to a different LDAP server");
-		return;
-	}
-
 	fr_connection_release(inst->pool, conn);
 	return;
 }
