@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: fcb3fd11fcedc0725ecc6280780867baa37c0421 $
+ * $Id: 9db0c0fb1eff4396cd91e1b1b7f34fc2a9c0a7f6 $
  *
  * @brief Functions and datatypes for the REST (HTTP) transport.
  * @file rest.c
@@ -23,7 +23,7 @@
  * @copyright 2012-2014  Arran Cudbard-Bell <a.cudbard-bell@freeradius.org>
  */
 
-RCSID("$Id: fcb3fd11fcedc0725ecc6280780867baa37c0421 $")
+RCSID("$Id: 9db0c0fb1eff4396cd91e1b1b7f34fc2a9c0a7f6 $")
 
 #include <ctype.h>
 #include <string.h>
@@ -1099,7 +1099,7 @@ static int rest_decode_post(UNUSED rlm_rest_t *instance, UNUSED rlm_rest_section
 	/*
 	 *	Empty response?
 	 */
-	while (isspace(*p)) p++;
+	while (isspace((uint8_t) *p)) p++;
 	if (*p == '\0') return 0;
 
 	while (((q = strchr(p, '=')) != NULL) && (count < REST_BODY_MAX_ATTRS)) {
@@ -1563,7 +1563,7 @@ static int rest_decode_json(rlm_rest_t *instance, rlm_rest_section_t *section,
 	/*
 	 *  Empty response?
 	 */
-	while (isspace(*p)) p++;
+	while (isspace((uint8_t) *p)) p++;
 	if (*p == '\0') return 0;
 
 	json = json_tokener_parse(p);
@@ -2080,8 +2080,12 @@ int rest_request_config(rlm_rest_t *instance, rlm_rest_section_t *section,
 	SET_OPTION(CURLOPT_CONNECTTIMEOUT_MS, instance->connect_timeout);
 	SET_OPTION(CURLOPT_TIMEOUT_MS, section->timeout);
 
-#ifdef CURLOPT_PROTOCOLS
+#if CURL_AT_LEAST_VERSION(7,85,0)
+	SET_OPTION(CURLOPT_PROTOCOLS_STR, "http,https");
+#else
+#  ifdef CURLOPT_PROTOCOLS
 	SET_OPTION(CURLOPT_PROTOCOLS, (CURLPROTO_HTTP | CURLPROTO_HTTPS));
+#  endif
 #endif
 
 	/*
@@ -2237,9 +2241,11 @@ int rest_request_config(rlm_rest_t *instance, rlm_rest_section_t *section,
 		SET_OPTION(CURLOPT_CAPATH, section->tls_ca_path);
 	}
 
+#if !CURL_AT_LEAST_VERSION(7,84,0)
 	if (section->tls_random_file) {
 		SET_OPTION(CURLOPT_RANDOM_FILE, section->tls_random_file);
 	}
+#endif
 
 	SET_OPTION(CURLOPT_SSL_VERIFYPEER, (section->tls_check_cert == true) ? 1 : 0);
 	SET_OPTION(CURLOPT_SSL_VERIFYHOST, (section->tls_check_cert_cn == true) ? 2 : 0);
